;*******************************************************************************************
;   Beschreibung:    Per Tastendruck knnen verschiedene Texte auf dem Display dargestellt *
;							werden.																					 *
;							Taste 1: "U=2.52V  I=2.58A"														 *
;										"P=6.60W  R=1.03_"														 *
;							Taste 2: "Mikrocontroller "														 *
;										"  PIC16F876A    "														 *
;							Taste 3: " Hello          "														 *
;										"       World!   "														 *
;							Taste 4: "Michael Hofmann "														 *
;										"  PIC16F876A    "														 *
;																														 *
;							ber die serielle Schnittstelle wird "Hello World!" ausgegeben			 *
;							(Einstellung:	Bits pro Sekunde = 8												 *
;												Datenbits = 8														 *
;												Paritt = keine													 *
;												Stoppbits = 1														 *
;												Flusssteuerung = kein											 *
;																														 *
;*******************************************************************************************
;                                                                                          *
;    Dateiname:	 	Hardwaretest.asm					                                        *
;    Datum:        	08.03.2009                                                            *
;    Version: 			V1.0                                                                  *
;                                                                                          *
;    Autor:       	Michael Hofmann                                                       *
;    Internet:      	www.edmh.de                                                           *
;    e-mail:			info@edmh.de                                                          *
;                                                                                          *
;*******************************************************************************************
;    Bemerkungen:                                                                          *
;    		Das Beispiel kann mit dem Entwicklungsboard untersucht werden.	   		 			 *
;			Debugger->Select Tool->MPLAB ICD 2               	                               *
;                                                                                          *
;*******************************************************************************************


	list      p=16f876a           ;definiert den verwendeten Prozessor
	#include <p16f876a.inc>       ;in diesem File stehen die Adressen und Bitnamen
	#include <Macros.inc>			;Makros fr die Bankumschaltung und die Zeitverzgerung
	#include <Display.inc>			;Makro fr die Initialisierung des Displays

	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_OFF & _XT_OSC & _WRT_OFF & _LVP_OFF & _DEBUG_ON & _CPD_OFF 
	; Mit '__CONFIG' werden die Configurationsbits gesetzt, z.B. Watchdog an/aus, Debugmodus

;===========================
;  DEFINITIONEN
;===========================
#define LED_1			PORTA, 2		;LED 1 liegt an Pin RA2
#define LED_3			PORTB, 4		;LED 3 liegt an Pin RB4
#define LED_4			PORTB, 5		;LED 4 liegt an Pin RB5
											;LED 2 wird muss wegen Register ADCON1
											;als analoger Eingang definiert sein

#define TASTER_1		PORTA, 4		;Taster 1 liegt an Pin RA4
#define TASTER_2		PORTA, 5		;Taster 2 liegt an Pin RA5
#define TASTER_3		PORTC, 0		;Taster 3 liegt an Pin RC0
#define TASTER_4		PORTC, 1		;Taster 4 liegt an Pin RC1

#define DISP_SI		PORTB, 0		;Serielle Daten zum Display
#define DISP_CLK		PORTB, 1		;Takt fr die Displaydaten
#define DISP_RS		PORTB, 2		;Unterscheidung zwischen Befehl und Daten
#define DISP_CSB		PORTB, 3		;Freigabe der Datenbertragung an das Display

;===========================
;  VARIABLEN
;===========================
w_temp        	EQU	0x70		;werden fr Interrupts verwendet, um den Status vor 
status_temp   	EQU	0x71		;der Interruptroutine zu sichern

DISP_SHIFT_REG	EQU	0x20		;Datum, welches an das Display gesendet werden soll
DISP_SHIFT_CNT	EQU	0x21		;Zhler fr die Datenausgabe

CALC_L_1			EQU	0x22		;1. Rechenregister fr Additionen und Subtraktionen
CALC_H_1			EQU	0x23		;High-Wort des CALC-Registers
CALC_L_2			EQU	0x24		;2. Rechenregister
CALC_H_2			EQU	0x25		;Bsp: CALC_x_1 + CALC_x_2 = CALC_x_1

COUNTER			EQU	0x26		;Zhler

ADC_L				EQU	0x30		;Kopie von AD-Wandlung
ADC_H				EQU	0x31	
ADC_Lx25			EQU	0x32		;AD-Wert mal 25
ADC_Hx25			EQU	0x33
ADC_Lx25_64		EQU	0x34		;AD-Wert mal 25/64
ADC_Hx25_64		EQU	0x35
ADC_Lx25_256	EQU	0x36		;AD-Wert mal 25/256
ADC_Hx25_256	EQU	0x37
Ux100_L			EQU	0x38		;Spannungswert mal 100
Ux100_H			EQU	0x39
DISP_HUNDERTER	EQU	0x3A		;Hunderterstelle
DISP_ZEHNER		EQU	0x3B		;Zehnerstelle
DISP_EINER		EQU	0x3C		;Einerstelle

MUL_1_H			EQU	0x3D		;1. Multiplikator (Highbyte)
MUL_1_L			EQU	0x3E		;1. Multiplikator (Lowbyte)
MUL_2_H			EQU	0x3F		;2. Multiplikator (Highbyte)
MUL_2_L			EQU	0x40		;2. Multiplikator (Lowbyte)
MUL_E_4			EQU	0x41		;hchstwertiges Byte des Produktes
MUL_E_3			EQU	0x42		;3. Byte des Produktes
MUL_E_2			EQU	0x43		;2. Byte des Produktes
MUL_E_1			EQU	0x44		;niederwertiges Byte des Produktes

ZAEHLER_H		EQU	0x45		;hchstwertiges Byte des Zhlers fr die Division
ZAEHLER_M		EQU	0x46		;mittelwertiges Byte des Zhlers
ZAEHLER_L		EQU	0x47		;niederwertigse Byte des Zhlers
NENNER_H			EQU	0x48		;hchstwertiges Byte des Nenners fr die Division
NENNER_L			EQU	0x49		;niederwertiges Byte des Nenners
DIV_COUNTER		EQU	0x4A		;Zhler fr die Stelle der Division
REST_H			EQU	0x4B		;hchstwertiges Byte des Restes
REST_L			EQU	0x4C		;niederwertiges Byte des Restes

MUL_X10_H		EQU	0x4D		;Rechenregister fr die Multiplikation mit 10 (Highbyte)
MUL_X10_M		EQU	0x4E		;(mittleres Byte)
MUL_X10_L		EQU	0x4F		;(Lowbyte)
MUL_X10_H_ERG	EQU	0x50
MUL_X10_M_ERG	EQU	0x51
MUL_X10_L_ERG	EQU	0x52

STELLE_1			EQU	0x53		;erste dezimale Stelle des Divisionsergebnisses
STELLE_2			EQU	0x54		;zweite dezimale Stelle
STELLE_3			EQU	0x55		;dritte dezimale Stelle
STELLE_4			EQU	0x56		;vierte dezimale Stelle
STELLE_5			EQU	0x57		;fnfte dezimale Stelle
STELLE_6			EQU	0x58		;sechste dezimale Stelle
STELLE_CNT		EQU	0x59		;wenn 0, wurden alle signifikanten Stellen ausgegeben

STROM_L			EQU	0x5A		;Stromwert multipliziert mit 100 (Highbyte)
STROM_H			EQU	0x5B		;Stromwert multipliziert mit 100 (Lowbyte)
SPG_L				EQU	0x5C		;Spannungswert multipliziert mit 100 (Highbyte)
SPG_H				EQU	0x5D		;Spannungswert multipliziert mit 100 (Lowbyte)

;__________________________________________________________________________________________

		ORG     0x000				; Adresse, an die nach einem Reset gesprungen wird
		clrf    PCLATH          ; Rcksetzen der Page-Bits
  		goto    start           ; Springe zum Programmanfang

		ORG     0x004           ; Adresse, an die nach einem Interrupt gesprungen wird
		movwf	  w_temp          ; Sichere den Inhalt des W-Registers
		movf    STATUS,w        ; Verschiebe STATUS-Register ins W-Register
		movwf	  status_temp     ; Sichere den Inhalt des STATUS-Registers

		; Ab hier kann der Code fr die Interruptroutinen stehen


		movf    status_temp,w     	; hole die Kopie des STATUS-Registers
		movwf	  STATUS            	; Sichere die Daten wieder zurck in das STATUS-Register
		swapf   w_temp,f
		swapf   w_temp,w          	; Sichere die Daten wieder zurck in das W-Register
		retfie                    	; Rcksprung vom Interrupt
;__________________________________________________________________________________________

start							; Beginn des Programms
	_DELAY_TMR1_US D'500000'
	; Initialisierungen
	_BANK_0
	clrf PORTA				;Lsche alle Ausgangsports
	clrf PORTB
	clrf PORTC
	_BANK_1
	movlw b'10000100'		;AN0, AN1 u. AN3 = analoger Eingang, Ergebnis rechtsbndig
	movwf ADCON1
	movlw b'11111011'		;Setze RA2 als Ausgang und den Rest als Eingang
	movwf TRISA
	movlw b'11000000'		;Setze RB7 und RB6 als Eingang und den Rest als Ausgang
	movwf TRISB
	movlw b'11111111'		;Alle Pins von Port C sind Eingnge
	movwf	TRISC
	_BANK_0
	movlw b'01000000'		;Initialisiere ADC, Kanal 0 (AN0), Fosz/8 (-> max 5 MHz Takt)
	movwf ADCON0

	bsf DISP_CSB			;Displaysignal CSB auf Highpegel setzen -> Display nicht aktiv
	bcf DISP_CLK			;Taktleitung auf Low setzen
	bcf DISP_SI				;Datenleitung auf Low setzen
	bcf DISP_RS				;Signalzustand von RS auf Kommando setzen

	_INIT_DISPLAY					;nach der Initialisierung ist der Inhalt des Displays
										;gelscht, bzw. mit Leerzeichen vollgeschrieben
	_DELAY_TMR1_US D'100000'

init_serial
	_BANK_1
	movlw D'12'				;setze Baudrate auf 19,2k (19.231 KBAUD)
	movwf SPBRG
	movlw B'00000100'		;setze 8-bit-Mode, High-Speed
	movwf TXSTA
	_BANK_0
	movlw B'10010000'		;starte Empfang
	movwf RCSTA

main								; Beginn der Hauptschleife
	btfss TASTER_1				;Abfrage, ob Taster 1 gedrckt wurde
	goto taster_1_gedrckt	;Schalte die LED's fr Taster 1
	btfss TASTER_2				;Abfrage, ob Taster 2 gedrckt wurde
	goto taster_2_gedrckt	;Schalte die LED's fr Taster 2
	btfss TASTER_3				;Abfrage, ob Taster 3 gedrckt wurde
	goto taster_3_gedrckt	;Schalte die LED's fr Taster 3
	btfss TASTER_4				;Abfrage, ob Taster 4 gedrckt wurde
	goto taster_4_gedrckt	;Schalte die LED's fr Taster 4
	bsf LED_1
	_DELAY_TMR1_US D'300000'
	bcf LED_1
	_DELAY_TMR1_US D'300000'

	movlw A'H'				;es werden die Zeichen "S3" mit Return gesendet
	call SendeZeichen
	movlw A'e'
	call SendeZeichen
	movlw A'l'
	call SendeZeichen
	movlw A'l'
	call SendeZeichen
	movlw A'o'
	call SendeZeichen
	movlw A' '
	call SendeZeichen
	movlw A'W'
	call SendeZeichen
	movlw A'o'
	call SendeZeichen
	movlw A'r'
	call SendeZeichen
	movlw A'l'
	call SendeZeichen
	movlw A'd'
	call SendeZeichen
	movlw A'!'
	call SendeZeichen
	movlw 0x0A				;steht fr LF = Linefeed = neue Zeile
	call SendeZeichen
	movlw 0x0D				;steht fr CR = Carrige Return = setzte Cursor auf Zeilenanfang
	call SendeZeichen

	goto main					;Kein Taster wurde gedrckt --> Fange von vorne an

taster_1_gedrckt									;Beginn der Hauptschleife
	movlw 0x80						;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call SchreibeDispKommando	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'U'						;Das Zeichen "U" soll in der 1. Zeile an Pos. 1 stehen
	call SchreibeDispDaten		;bertrage das Zeichen
	movlw A'='						;Lade das Zeichen "=" in das W-Register
	call SchreibeDispDaten		;automatische Adresserhhung "=" = 1. Zeile 2. Stelle

lese_AN0								;Einlesen der Spannung an Pin AN0
	_BANK_1
	clrf ADRESL						;niederwertigen Teil des AD-Ergebnisses lschen
	_BANK_0
	clrf ADRESH						;hherwertigen Teil des AD-Ergebnisses lschen
	movlw b'01000000'				;Initialisiere ADC, Kanal 0 (AN0), Fosz/8 (-> max 5 MHz Takt)
	movwf ADCON0
	bsf ADCON0, ADON				;AD-Modul anschalten
	_DELAY_TMR1_US d'40'			;Wartezeit, damit sich der Haltekondensator aufladen kann
	bsf ADCON0, GO_DONE			;AD-Wandlung starten
	btfsc ADCON0, GO_DONE
	goto $-1							;Warten, bis das bit GO_DONE von der Hardware zurckgesetzt
	nop								;wurde -> AD-Wandlung beendet
	
	call AD_konvertieren			;Umwandlung AD-Ergebnis in Spannungswert
	movf Ux100_L, W				;Den Spannungswert ins W-Register holen
	movwf MUL_1_L					;und in die Register MUL_1_L und SPG_L kopieren
	movwf SPG_L						;wird fr die sptere Berechung der Leistung bentigt
	movf Ux100_H, W				;Das Highbyte des Spannungswertes in die Register
	movwf MUL_1_H					;MUL_1_H und SPG_H kopieren
	movwf SPG_H						;wird fr die sptere Berechnung der Leistung bentigt
	call B2D							;Umwandlung des binren Wertes in 3 Dezimalstellen

	movlw 0x82						;Pos. 3 der 1. Zeile
	call SchreibeDispKommando
	movlw 0x30						;Umwandlung der Hunderterstelle in ASCII
	addwf DISP_HUNDERTER, W		;ASCII-Code = Zahlenwert + 0x30
	call SchreibeDispDaten		;Ausgabe der Stelle vor dem Komma
	movlw A'.'
	call SchreibeDispDaten		;Ausgabe des Dezimalpunktes
	movlw 0x30						;Umwandlung der Zehnerstelle in ASCII
	addwf DISP_ZEHNER, W
	call SchreibeDispDaten		;Ausgabe der ersten Stelle hinter dem Komma
	movlw 0x30						;Umwandlung der Einerstelle in ASCII
	addwf DISP_EINER, W
	call SchreibeDispDaten		;Ausgabe der zweiten Stelle hinter dem Komma
	movlw A'V'
	call SchreibeDispDaten		;Ausgabe der Spannungseinheit
	movlw A' '
	call SchreibeDispDaten		;Ausgabe der Spannungseinheit
	movlw A' '
	call SchreibeDispDaten		;Ausgabe der Spannungseinheit

	;movlw 0x89						;Zeichen 10 in der 1. Zeile (=0x80 + 0x09)
	;call SchreibeDispKommando	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'I'						;Das Zeichen "I" soll in der 1. Zeile an Pos. 1 stehen
	call SchreibeDispDaten		;bertrage das Zeichen
	movlw A'='						;Lade das Zeichen "=" in das W-Register
	call SchreibeDispDaten		;automatische Adresserhhung "=" = 1. Zeile 11. Stelle

lese_AN1								;Einlesen der Spannung an Pin AN1 (Strom)
	_BANK_1
	clrf ADRESL						;niederwertigen Teil des AD-Ergebnisses lschen
	_BANK_0
	clrf ADRESH						;hherwertigen Teil des AD-Ergebnisses lschen
	movlw b'01001000'				;Initialisiere ADC, Kanal 1 (AN1), Fosz/8 (-> max 5 MHz Takt)
	movwf ADCON0
	bsf ADCON0, ADON				;AD-Modul anschalten
	_DELAY_TMR1_US d'40'			;Wartezeit, damit sich der Haltekondensator aufladen kann
	bsf ADCON0, GO_DONE			;AD-Wandlung starten
	btfsc ADCON0, GO_DONE
	goto $-1							;Warten, bis das bit GO_DONE von der Hardware zurckgesetzt
	nop								;wurde -> AD-Wandlung beendet
	
	call AD_konvertieren			;Umwandlung AD-Ergebnis in Spannungswert
	movf Ux100_L, W				;Lowbyte des konvertierten Ergebnisses in die Register
	movwf MUL_2_L					;MUL_2_L und STROM_L kopieren
	movwf STROM_L					;wird fr die sptere Berechung der Leistung bentigt
	movf Ux100_H, W				;Highbyte der konvertierten Ergebnisse in die Register
	movwf MUL_2_H					;MUL_2_H und STROM_H kopieren
	movwf STROM_H					;wird fr die sptere Berechung der Leistung bentigt
	call B2D							;Umwandlung des binren Wertes in 3 Dezimalstellen

	movlw 0x8B						;Pos. 13 der 1. Zeile
	call SchreibeDispKommando
	movlw 0x30						;Umwandlung der Hunderterstelle in ASCII
	addwf DISP_HUNDERTER, W		;ASCII-Code = Zahlenwert + 0x30
	call SchreibeDispDaten		;Ausgabe der Vorkommastelle
	movlw A'.'
	call SchreibeDispDaten		;Ausgabe des Dezimalpunktes
	movlw 0x30						;Umwandlung der Zehnerstelle in ASCII
	addwf DISP_ZEHNER, W
	call SchreibeDispDaten		;Ausgabe der ersten Stelle hinter dem Komma
	movlw 0x30						;Umwandlung der Einerstelle in ASCII
	addwf DISP_EINER, W
	call SchreibeDispDaten		;Ausgabe der zweiten Stelle hinter dem Komma
	movlw A'A'
	call SchreibeDispDaten		;Ausgabe der Stromeinheit

	;----------------------------------
	;Berechung der Leistung (P = U * I)
	;----------------------------------
	movlw 0xC0						;Zeichen 1 in der 2. Zeile (=0xC0 + 0x00)
	call SchreibeDispKommando	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'P'						;Das Zeichen "P" soll in der 2. Zeile an Pos. 1 stehen
	call SchreibeDispDaten		;bertrage das Zeichen
	movlw A'='						;Lade das Zeichen "=" in das W-Register
	call SchreibeDispDaten		;automatische Adresserhhung "=" = 2. Zeile 2. Stelle

	call Mul16						;Berechnung der Leistung. Die beiden Multiplikatoren
	nop								;stehen bereits in den Registern MUL_1_H/L und MUL_2_H/L
	movf MUL_E_3, W				;Da der Stromwert und der Spannungswert zuvor jeweils mit
	movwf ZAEHLER_H				;100 multipliziert wurden hat die Leistung in den Registern
	movf MUL_E_2, W				;MUL_E_3 bis MUL_E_1 den 10000-fachen Wert. Um auf den
	movwf ZAEHLER_M				;richtigen Wert zu kommen wird der Leistungswert mehrfach
	movf MUL_E_1, W				;durch 10 dividiert. Dazu werden zuerst die Zhlerregister
	movwf ZAEHLER_L				;mit dem Ergebnis der Multiplikation geladen.
	
	clrf NENNER_H					;Bestimmung der einzelnen Stellen
	movlw d'10'						;Das Ergebnis der Leistungsberechnung wird durch 10 geteilt.
	movwf NENNER_L
	call Div16						;Ausfhrung der ersten Division, um die niederwertigste 
	movf REST_L, W					;Stelle zu bestimmen. Der Rest der Division ist der Wert
	movwf STELLE_6					;fr die 4. Nachkommastelle und wird in Register STELLE_6
	call Div16						;gespeichert
	movf REST_L, W
	movwf STELLE_5					;Speichern der 3. Nachkommastelle
	call Div16
	movf REST_L, W
	movwf STELLE_4					;Speichern der 2. Nachkommastelle
	call Div16
	movf REST_L, W
	movwf STELLE_3					;Speichern der 1. Nachkommastelle
	call Div16
	movf REST_L, W
	movwf STELLE_2					;Speichern der 1. Stelle vor dem Komma (Einer)
	call Div16
	movf REST_L, W
	movwf STELLE_1					;Speichern der 2. Stelle vor dem Komma (Zehner)
	nop
prfe_Stelle1_P					;Fr die Anzeige auf dem Display sollen nur 3 Stellen
	movlw d'3'						;verwendet werden. Das Ergebnis wird nicht gerundet!
	movwf STELLE_CNT				;Zhler fr die Anzahl der Stellen laden.
	movlw 0xFF
	andwf STELLE_1, W				;Prfen, ob die erste Stelle 0 ist
	btfsc STATUS, Z				;Fhrende Nullen sollen nicht ausgegeben werden
	goto prfe_Stelle2_P
schreibe_Stelle1_P				;Die erste Stelle war grer als 0
	movlw 0x30						;Umwandlung der Zehnerstelle in ASCII
	addwf STELLE_1, W
	call SchreibeDispDaten		;Ausgabe auf dem Display
	decfsz STELLE_CNT				;es wurde eine Stelle ausgegeben -> Zhler verringern
	goto schreibe_Stelle2_P		;Wenn der Zhler > 0 ist wird die nchste Stelle ausgegeben
	goto schreibe_Einheit_P		;Alle Stellen ausgegeben -> Ausgabe der Einheit

prfe_Stelle2_P
	movlw 0xFF
	andwf STELLE_2, W				;prfen, ob die zweite Stelle > 0 ist
	btfsc STATUS, Z
	goto prfe_Stelle3_P			;ist die Stelle = 0 wird die dritte Stelle geprft
schreibe_Stelle2_P				;2. Stelle ist > 0 und wird ausgegeben
	movlw 0x30						;Umwandlung der Zehnerstelle in ASCII
	addwf STELLE_2, W
	call SchreibeDispDaten

	movlw A'.'						;Die Leistung wurde mit 4 Nachkommastellen berechnet, da
	call SchreibeDispDaten		;der Strom und die Spannung zuvor jeweils mit 100
	decfsz STELLE_CNT				;multipliziert wurden. Daher muss an dieser Stelle der
	goto schreibe_Stelle3_P		;Dezimalpunkt ausgegeben werden. Danach wird die 3. Stelle
	goto schreibe_Einheit_P		;ausgegeben oder die Einheit, falls der Zhler 0 ist

prfe_Stelle3_P					;Wenn die Stelle 2 zuvor 0 war, gibt es keine Stelle vor dem
	movlw A'.'						;Komma und der Dezimalpunkt ist das erste Zeichen.
	call SchreibeDispDaten		;Ausgabe des Dezimalpunktes.

schreibe_Stelle3_P				;Ab der 3. Stelle werden alle Ziffern ausgegeben
	movlw 0x30						;Umwandlung der Zehnerstelle in ASCII
	addwf STELLE_3, W
	call SchreibeDispDaten
	decfsz STELLE_CNT				;Wenn bereits 3 Ziffern ausgegeben wurden, wird die Einheit
	goto schreibe_Stelle4_P		;ausgegeben, ansonsten wird die 4. Stelle ausgegeben.
	goto schreibe_Einheit_P

schreibe_Stelle4_P				;Ausgabe der 2. Nachkommastelle
	movlw 0x30						;Umwandlung der Zehnerstelle in ASCII
	addwf STELLE_4, W
	call SchreibeDispDaten
	decfsz STELLE_CNT				;Falls noch eine 3. Nachkommastelle ausgegeben werden muss,
	goto schreibe_Stelle5_P		;wird die 5. Stelle auf das Display geschrieben.
	goto schreibe_Einheit_P		;Ausgabe der Einheit, wenn der Zhler fr die Stellen 0 ist.

schreibe_Stelle5_P				;3. Nachkommastelle schreiben
	movlw 0x30						;Umwandlung der Zehnerstelle in ASCII
	addwf STELLE_5, W				;Es sollen nur maximal 3 signifikante Stellen ausgegeben
	call SchreibeDispDaten		;werden, daher ist die 5. Stelle die letzte Stelle. Die
										;6. Stelle muss nicht mehr bercksichtigt werden.
schreibe_Einheit_P				;Nach der letzten signifikanten Stelle wird die Einheit
	movlw A'W'						;fr die Leistung ausgegeben.
	call SchreibeDispDaten
	nop
	;--------------------------------------
	;Berechung des Widerstandes (R = U / I)
	;--------------------------------------
	nop
	movf STROM_L, W				;Der Wert des Stromes passt in ein 16 bit Register
	movwf NENNER_L					;und wird in die beiden Register fr den Nenner kopiert.
	movf STROM_H, W
	movwf NENNER_H

	movf SPG_L, W					;Um auch kleine Widerstnde (unter 1 Ohm) anzeigen zu knnen 
	movwf MUL_X10_L				;muss der Spannungswert mit dem Faktor 1000 multipliziert
	movf SPG_H, W					;werden. Auf diese Weise kann man 3 Nachkommastellen
	movwf MUL_X10_M				;berechnen. Dazu wird der Spannungswert in das
	clrf MUL_X10_H					;Multiplikationsregister MUL_X10_L bis MUL_X10_H kopiert.
	nop
	call Mul_x10					;Der Spannungswert wird 3 Mal mit 10 multipliziert
	call Mul_x10					;10*10*10=1000
	call Mul_x10					;Der kleinste Widerstand ist 0.001 V / 5.00 A = 0.002 Ohm
	nop								;Der grte Widerstand ist 5.00 V / 0.01 A = 500 Ohm
	movf MUL_X10_L_ERG, W		;Kopieren des 1000-fachen Spannungswertes fr die Division.
	movwf ZAEHLER_L				;Der Spannungswert ist maximal 500000 und passt daher in
	movf MUL_X10_M_ERG, W		;das 24 bit Register fr die Division
	movwf ZAEHLER_M
	movf MUL_X10_H_ERG, W
	movwf ZAEHLER_H
	nop
	call Div16						;Durch die Division der Zhlerregister (Spannung) mit den
	nop								;Nennerregistern (Strom) erhlt man den Widerstand.
	clrf NENNER_H					;Das Ergebnis der Division steht in den Zhlerregistern.
	movlw d'10'						;Um die einzelnen Stellen des Widerstandwertes zu bestimmen,
	movwf NENNER_L					;wird das Ergebnis schrittweise durch 10 geteilt.
	call Div16						;Der jeweilige Rest wird in das Register fr die
	movf REST_L, W					;entsprechende Stelle geschrieben. 
	movwf STELLE_6					;3. Nachkommastelle
	call Div16
	movf REST_L, W
	movwf STELLE_5					;2. Nachkommastelle
	call Div16
	movf REST_L, W
	movwf STELLE_4					;1. Nachkommastelle
	call Div16
	movf REST_L, W
	movwf STELLE_3					;1. Stelle vor dem Komma (Einer)
	call Div16
	movf REST_L, W
	movwf STELLE_2					;2. Stelle vor dem Komma (Zehner)
	call Div16
	movf REST_L, W
	movwf STELLE_1					;3. Stelle vor dem Komma (Hunderter)
	nop
	;movlw 0xC9						;Zeichen 10 in der 2. Zeile (=0xC0 + 0x09)
	;call SchreibeDispKommando	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A' '
	call SchreibeDispDaten		;Ausgabe der Spannungseinheit
	movlw A' '
	call SchreibeDispDaten		;Ausgabe der Spannungseinheit
	movlw A'R'						;Das Zeichen "R" soll in der 2. Zeile an Pos. 10 stehen
	call SchreibeDispDaten		;bertrage das Zeichen
	movlw A'='						;Lade das Zeichen "=" in das W-Register
	call SchreibeDispDaten		;automatische Adresserhhung "=" = 2. Zeile 11. Stelle
prfe_Stelle1
	movlw d'3'						;Es sollen 3 Stellen ausgegeben werden
	movwf STELLE_CNT				;Zhler fr die Anzahl der signifikanten Stellen.
	movlw 0xFF						
	andwf STELLE_1, W				;Prfen, ob die Hunderter-Stelle 0 ist.
	btfsc STATUS, Z				;Wenn die Stelle 0 ist wird die fhrende Null nicht auf
	goto prfe_Stelle2			;dem Display ausgegeben.
schreibe_Stelle1					
	movlw 0x30						;Umwandlung der Hunderterstelle in ASCII
	addwf STELLE_1, W
	call SchreibeDispDaten		;Ausgabe auf dem Display
	decfsz STELLE_CNT				;Stellenzhler verringern nach der Ausgabe einer
	goto schreibe_Stelle2		;signifikanten Stelle. Wenn der Zhler > 0 ist, wird die
	goto schreibe_Einheit		;2. Stelle ausgegeben.

prfe_Stelle2						;Falls die erste Stelle 0 war, wird hier geprft, ob auch die
	movlw 0xFF						;zweite Stelle 0 ist.
	andwf STELLE_2, W
	btfsc STATUS, Z				;Ist die zweite Stelle auch Null, wird mit der Prfung
	goto prfe_Stelle3			;der dritten Stelle fortgefahren.
schreibe_Stelle2
	movlw 0x30						;Umwandlung der Zehnerstelle in ASCII
	addwf STELLE_2, W				
	call SchreibeDispDaten		;Die zweite Stelle vor dem Komma wird ausgegeben.
	decfsz STELLE_CNT				;Wenn der Zhler noch grer als 0 ist, wird die
	goto schreibe_Stelle3		;Einerstelle geschrieben, ansonsten wird die Einheit
	goto schreibe_Einheit		;ausgegeben.

prfe_Stelle3
	movlw 0xFF						;Wenn alle vorherigen Stellen 0 waren wird hier geprft,
	andwf STELLE_3, W				;ob auch die Einerstelle 0 ist. Ist diese auch Null
	btfsc STATUS, Z				;ist der Widerstand kleiner als 1 Ohm und es wird an die
	goto prfe_Stelle4			;nchste Stelle gesprungen.
schreibe_Stelle3
	movlw 0x30						;Umwandlung der Einerstelle in ASCII
	addwf STELLE_3, W
	call SchreibeDispDaten		;Ausgabe der Einerstelle auf dem Display.
	movlw A'.'						;Nach der Einerstelle (3.Stelle) folgen nur noch 
	call SchreibeDispDaten		;Nachkommastellen, daher wird an dieser Stelle auch der
	decfsz STELLE_CNT				;Dezimalpunkt ausgegeben.
	goto schreibe_Stelle4		;Wenn schon 3 signifikante Stellen ausgegeben wurden, wird
	goto schreibe_Einheit		;anschlieend die Einheit auf dem Display dargestellt.

prfe_Stelle4						;Wenn alle Stellen vor dem Komma 0 waren, gibt es nur noch
	movlw A'.'						;Nachkommastellen und der Dezimalpunkt wird ausgegeben.
	call SchreibeDispDaten

schreibe_Stelle4					;Ab der 4. Stelle mssen alle Ziffern ausgegeben werden,
	movlw 0x30						;daher entfllt die Prfung, ob die Stelle 0 ist.
	addwf STELLE_4, W
	call SchreibeDispDaten		;Ausgabe der ersten Nachkommastelle
	decfsz STELLE_CNT				;Prfen, ob noch weitere Stellen ausgegeben werden mssen
	goto schreibe_Stelle5		;oder ob die Einheit angefgt werden kann.
	goto schreibe_Einheit

schreibe_Stelle5
	movlw 0x30						;Umwandlung der 2. Nachkommastelle in ASCII
	addwf STELLE_5, W
	call SchreibeDispDaten		;Ausgabe der 2. Nachkommastelle
	decfsz STELLE_CNT				;Der Zhler fr die Stellen wird verringert und so geprft,
	goto schreibe_Stelle6		;ob noch eine Ziffer ausgegeben werden muss.
	goto schreibe_Einheit

schreibe_Stelle6
	movlw 0x30						;Umwandlung der 3. Nachkommastelle in ASCII
	addwf STELLE_6, W
	call SchreibeDispDaten		;Ausgabe der letzten Nachkommastelle. Diese steht fr mOhm.
schreibe_Einheit
	movlw 0xCF						;Das Zeichen fr die Einheit wird nach der letzten Ziffer
	call SchreibeDispKommando	;ausgegeben und steht in der 2. Zeile an Position 16.
	movlw 0x1E						;Ohmzeichen
	call SchreibeDispDaten

	_DELAY_TMR1_US D'200000'
	goto main						;Die Messung wird wieder neu gestartet

taster_2_gedrckt
	movlw 0x80						;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call SchreibeDispKommando	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'M'
	call SchreibeDispDaten
	movlw A'i'						;das Zeichen "H" soll in der 1. Zeile an Pos. 1 stehen
	call SchreibeDispDaten		;bertrage das Zeichen
	movlw A'k'						;lade das Zeichen "e" in das W-Register
	call SchreibeDispDaten		;automatische Adresserhhung "e" = 1. Zeile 2. Stelle
	movlw A'r'						
	call SchreibeDispDaten		;bertrage das Zeichen "l" an das Display
	movlw A'o'						
	call SchreibeDispDaten		;bertrage das zweite "l"
	movlw A'c'
	call SchreibeDispDaten		;Letzter Buchstabe der 1. Zeile = "o"
	movlw A'o'
	call SchreibeDispDaten
	movlw A'n'
	call SchreibeDispDaten
	movlw A't'
	call SchreibeDispDaten
	movlw A'r'
	call SchreibeDispDaten
	movlw A'o'
	call SchreibeDispDaten
	movlw A'l'
	call SchreibeDispDaten
	movlw A'l'
	call SchreibeDispDaten
	movlw A'e'
	call SchreibeDispDaten
	movlw A'r'
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten

	movlw 0xC0						;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call SchreibeDispKommando	;ndern der DDRAM-Adresse.
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A'P'
	call SchreibeDispDaten
	movlw A'I'
	call SchreibeDispDaten
	movlw A'C'
	call SchreibeDispDaten
	movlw A'1'
	call SchreibeDispDaten
	movlw A'6'
	call SchreibeDispDaten
	movlw A'F'						;Die Zeichen des Wortes "World" werden nacheinander
	call SchreibeDispDaten		;an das Display bertragen
	movlw A'8'
	call SchreibeDispDaten
	movlw A'7'
	call SchreibeDispDaten
	movlw A'6'
	call SchreibeDispDaten
	movlw A'A'
	call SchreibeDispDaten
	movlw A' '						;Als letztes Zeichen in der 2. Zeile wird noch das
	call SchreibeDispDaten		;Ausrufezeichen bertragen
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	_DELAY_TMR1_US D'200000'
	goto main

taster_3_gedrckt
	movlw 0x80						;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call SchreibeDispKommando	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A' '
	call SchreibeDispDaten
	movlw A'H'						;das Zeichen "H" soll in der 1. Zeile an Pos. 1 stehen
	call SchreibeDispDaten		;bertrage das Zeichen
	movlw A'e'						;lade das Zeichen "e" in das W-Register
	call SchreibeDispDaten		;automatische Adresserhhung "e" = 1. Zeile 2. Stelle
	movlw A'l'						
	call SchreibeDispDaten		;bertrage das Zeichen "l" an das Display
	movlw A'l'						
	call SchreibeDispDaten		;bertrage das zweite "l"
	movlw A'o'
	call SchreibeDispDaten		;Letzter Buchstabe der 1. Zeile = "o"
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten

	movlw 0xC0						;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call SchreibeDispKommando	;ndern der DDRAM-Adresse.
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A'W'						;Die Zeichen des Wortes "World" werden nacheinander
	call SchreibeDispDaten		;an das Display bertragen
	movlw A'o'
	call SchreibeDispDaten
	movlw A'r'
	call SchreibeDispDaten
	movlw A'l'
	call SchreibeDispDaten
	movlw A'd'
	call SchreibeDispDaten
	movlw A'!'						;Als letztes Zeichen in der 2. Zeile wird noch das
	call SchreibeDispDaten		;Ausrufezeichen bertragen
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	_DELAY_TMR1_US D'200000'
	goto main

taster_4_gedrckt
	movlw 0x80						;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call SchreibeDispKommando	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'M'
	call SchreibeDispDaten
	movlw A'i'						;das Zeichen "H" soll in der 1. Zeile an Pos. 1 stehen
	call SchreibeDispDaten		;bertrage das Zeichen
	movlw A'c'						;lade das Zeichen "e" in das W-Register
	call SchreibeDispDaten		;automatische Adresserhhung "e" = 1. Zeile 2. Stelle
	movlw A'h'						
	call SchreibeDispDaten		;bertrage das Zeichen "l" an das Display
	movlw A'a'						
	call SchreibeDispDaten		;bertrage das zweite "l"
	movlw A'e'
	call SchreibeDispDaten		;Letzter Buchstabe der 1. Zeile = "o"
	movlw A'l'
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A'H'
	call SchreibeDispDaten
	movlw A'o'
	call SchreibeDispDaten
	movlw A'f'
	call SchreibeDispDaten
	movlw A'm'
	call SchreibeDispDaten
	movlw A'a'
	call SchreibeDispDaten
	movlw A'n'
	call SchreibeDispDaten
	movlw A'n'
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten

	movlw 0xC0						;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call SchreibeDispKommando	;ndern der DDRAM-Adresse.
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A'P'
	call SchreibeDispDaten
	movlw A'I'
	call SchreibeDispDaten
	movlw A'C'
	call SchreibeDispDaten
	movlw A'1'
	call SchreibeDispDaten
	movlw A'6'
	call SchreibeDispDaten
	movlw A'F'						;Die Zeichen des Wortes "World" werden nacheinander
	call SchreibeDispDaten		;an das Display bertragen
	movlw A'8'
	call SchreibeDispDaten
	movlw A'7'
	call SchreibeDispDaten
	movlw A'6'
	call SchreibeDispDaten
	movlw A'A'
	call SchreibeDispDaten
	movlw A' '						;Als letztes Zeichen in der 2. Zeile wird noch das
	call SchreibeDispDaten		;Ausrufezeichen bertragen
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	movlw A' '
	call SchreibeDispDaten
	_DELAY_TMR1_US D'200000'
	goto main

;__________________________________________________________________________________________

;===========================
;  Unterprogramme
;===========================


;*******************************************************************************************
; Multiplikation eines 24bit Wertes mit 10
; Der Wert muss wieder in ein 24 bit breites Register passen, der grte Wert
; ist daher: 16777216 / 10 = 1677721
; MUL_X10_x * 10 = MUL_X10_x
;
;*******************************************************************************************
Mul_x10
	movf MUL_X10_L, W			;Eingangsdaten kopieren
	movwf MUL_X10_L_ERG		;In den Registern MUL_X10_L/M/H_ERG werden die Zwischenergebnisse
	movf MUL_X10_M, W			;gespeichert und bearbeitet
	movwf MUL_X10_M_ERG
	movf MUL_X10_H, W
	movwf MUL_X10_H_ERG

	movlw d'3'					;Die Ergebnisregister sollen 3 mal nach links geschoben werden,
	movwf COUNTER				;um eine Multiplikation mit 8 zu erreichen
mul_x2_loop
	bcf STATUS, C				;Carrybit lschen, damit es nicht die Multiplikation verflscht
	rlf MUL_X10_L_ERG, F		;Multiplikation mit 2
	rlf MUL_X10_M_ERG, F
	rlf MUL_X10_H_ERG, F
	decfsz COUNTER, F
	goto mul_x2_loop			;es wurde noch nicht dreimal mit 2 multipliziert

	movlw d'2'					;Der Wert ist nun mit 8 Multipliziert und es muss noch 2 mal
	movwf COUNTER				;der Zahlenwert addiert werden
add_loop
	movf MUL_X10_L, W			;Addition des unteren Bytes
	addwf MUL_X10_L_ERG, F
	btfsc STATUS, C			;prfen, ob ein bertrag aufgetreten ist
	incf MUL_X10_M_ERG, F	;es ist ein bertrag vorhanden
	movf MUL_X10_M, W			;Addition des mittleren Bytes
	addwf MUL_X10_M_ERG, F
	btfsc STATUS, C			;prfen, ob ein bertrag aufgetreten ist
	incf MUL_X10_H_ERG, F	;es ist ein bertrag vorhanden
	movf MUL_X10_H, W			;Addition des hchsten Bytes
	addwf MUL_X10_H_ERG, F

	decfsz COUNTER, F
	goto add_loop				;Der wert muss noch einmal addiert werden

	movf MUL_X10_L_ERG, W	;Ergebnis in das Eingangsregister kopieren, damit man leichter
	movwf MUL_X10_L			;eine mehrfache Multiplikation mit 10 machen kann
	movf MUL_X10_M_ERG, W
	movwf MUL_X10_M
	movf MUL_X10_H_ERG, W
	movwf MUL_X10_H

	return


;*******************************************************************************************
; Division eines 24 bit Wertes durch einen 16 bit Wert das Ergebnis wird in die
; Register des Zhlers gespeichert.
;
; Zhler / Nenner = Zhler + Rest 
;
; Verwendete Register:
; Zhler (24 bit) - ZAEHLER_H:ZAEHLER_M:ZAEHLER_L
; Nenner (16 bit) - NENNER_H:NENNER_L
;
; Zhler (8 bit)	- DIV_COUNTER
; Rest (16 bit)	- REST_H:REST_L
;
; Ergebnis (24 bit) - ZAEHLER_H:ZAEHLER_M:ZAEHLER_L
;
;*******************************************************************************************
Div16
	clrf REST_H					;Rest von vorherigen Berechnungen lschen
	clrf REST_L					
	movlw d'24'					;DIV_COUNTER mit 24 laden, da 24 bit im Zhler
	movwf DIV_COUNTER
div16_loop
	rlf ZAEHLER_L, F        ;den gesamtem Zhler um 1 nach links schieben
	rlf ZAEHLER_M, F
	rlf ZAEHLER_H, F

	rlf REST_L, F				;das Register fr den Rest um 1 nach links schieben
	rlf REST_H, F

	rlf DIV_COUNTER, F		;das Carrybit wird vorbergehend im DIV_COUNTER gespeichert
									;dadurch kann man sich ein Register sparen	
	movf NENNER_L, W
	subwf REST_L, F			;Subtrahiere Nenner_L von Rest_L (REST_L-NENNER_L=REST_L)
	movf NENNER_H, W
	btfss STATUS, C
	incfsz NENNER_H, W
	subwf REST_H, W			;Subtrahiere Nenner_H von Rest_H (REST_H-NENNER_H=W-Reg)
	
	btfsc STATUS, C			;prfe, ob das Ergebnis positiv ist
	bsf DIV_COUNTER, 0		;wenn ja, setzte gespeichertes Carrybit in DIV_COUNTER
	
	btfsc DIV_COUNTER, 0		;prfe das gespeicherte Carrybit in DIV_COUNTER
	goto div16_jump         ;wenn Carry=1 springe nach div16_jump
	
	movf NENNER_L, W			;Subtraktion wieder rckgngig machen, wenn das Ergebnis
	addwf REST_L, F			;negativ war und ein bertrag aufgetreten ist
	movf REST_H, W

div16_jump
	movwf REST_H				;Speichere den Rest vom W-Register in REST_H
	bcf STATUS, C				;Carrybit lschen und das gesicherte Carrybit in Register
	rrf DIV_COUNTER, F		;DIV_COUNTER wieder zurcksichern.
	decfsz DIV_COUNTER, F   ;DIV_COUNTER um 1 verringern
	goto div16_loop         ;wenn der Counter nicht null ist wird die nchste
	 
	rlf ZAEHLER_L, F			;gesamter Zhler noch einmal nach links schieben, damit
	rlf ZAEHLER_M, F			;das richtige Ergebnis im Zhler steht
	rlf ZAEHLER_H, F
	return
	
;*******************************************************************************************
; 16 bit Multiplikation
; Die Werte fr die Multiplikation mssen in den Registern MUL_1_H/L und MUL_2_H/L stehen
; Das Ergebnis steht dann in den Registern MUL_E_4 bis MUL_E_1
; MUL_1_[H/L] * MUL_2_[H/L] = MUL_E_[4..1]
;*******************************************************************************************
Mul16
	clrf MUL_E_4				;Lschen des vorherigen Ergebnisses
	clrf MUL_E_3
	clrf MUL_E_2
	clrf MUL_E_1
	bsf MUL_E_2, d'7'			;um zu erkennen, wann 16 mal geschoben wurde
mul16_loop
	rrf MUL_1_H, F				;verschiebt MUL_1 um nach rechts
	rrf MUL_1_L, F				;wenn das LSB eine 1 ist wird das Carry gesetzt und es muss
	btfss STATUS, C			;eine Addition durchgefhrt werden ist das Carry = 0 entspricht
	goto mul16_rotate			;dies einer Multiplikation mit 0 und es wird nichts addiert
	movf MUL_2_L, W			;Beginn der 16 bit Addition
	addwf MUL_E_3, F
	movf MUL_2_H, W
	btfsc STATUS, C			;prft, ob ein bertrag vorhanden ist
	incfsz MUL_2_H, W			;es ist ein bertrag vorhanden und das Highbyte wird um 1 erhht
	addwf MUL_E_4, F			;kein bertrag, das Highbyte wird addiert
mul16_rotate
	rrf MUL_E_4, F				;Das 32 Bit Ergebnisregister wird um 1 nach rechts geschoben
	rrf MUL_E_3, F
	rrf MUL_E_2, F
	rrf MUL_E_1, F
	btfss STATUS, C			;ist das Bit 7 von MUL_E_2 im Carry angekommen wurde das Ergebnis
	goto mul16_loop			;insgesamt 16 mal rotiert und die Berechnung ist beendet

	return                  ; fertig


;*******************************************************************************************
; 16 bit Addition
; Die Werte fr die Addition mssen in den Registern CALC_x_x stehen
; Das Ergebnis steht dann im Register CALC_x_1
; CALC_x_1 + CALC_x_2 = CALC_x_1
;*******************************************************************************************
Add16
	movf CALC_L_2, W		;Hole das Lowbyte des 2. Summanden
	addwf CALC_L_1, F		;Addiere den 1. und 2. Summanden
	btfsc STATUS, C		;Prfe, ob ein berlauf aufgetreten ist
	incf CALC_H_1, F		;wenn ja, Highbyte um 1 erhhen
	movf CALC_H_2, W		;Hole das Highbyte des 2. Summanden
	addwf CALC_H_1, F    ;Addiere die Highbytes der Summanden

	return                  ; fertig

;*******************************************************************************************
; 16 Bit Subtraktion
; Wenn das Ergebnis ein negativer Wert ist, ist das Carryflag auf 0 gesetzt
; CALC_x_1 - CALC_x_2 = CALC_x_1
;*******************************************************************************************
Sub16
	movf CALC_L_2, W		;Hole das Lowbyte des Subtrahenden
	subwf CALC_L_1, F		;CALC_L_1 - CALC_L_2 = CALC_L_1
	btfss STATUS, C		;Ist ein bertrag aufgetreten
	incf CALC_H_2			;Ja, erhhe das Highbyte um 1
	movf CALC_H_2, W		;Hole das Highbyte des Subtrahenden
	subwf CALC_H_1, F		;CALC_H_1 - CALC_H_2 = CALC_H_2

	return

;*******************************************************************************************
; Wandlung einer Binrzahl (< 1000) aus der AD-Wandlung in eine Dezimalzahl
; Die Binrzahl steht in den Registern Ux100_H und Ux100_L
; die Dezimalstellen werden in den folgenden Registern gespeichert:
;		DISP_HUNDERER = Hunderterstelle (Stelle vor dem Komma)
;		DISP_ZEHNER   = Zehnerstelle (erste Stelle nach dem Komma)
;		DISP_EINER    = Einerstelle (zweite Stelle nach dem Komma)
;Bsp: 0x0141 = 321 -> Hunderter = 3, Zehner = 2, Einer = 1
;Um die Division durch 100 und 10 zu realisieren, wird der entsprechende Wert (z.B. 100)
;sooft von dem Spannunswert abgezogen, bis das Ergebnis negativ ist. Wenn das Ergebnis
;negativ ist, wurde zuviel abgezogen und der Wert muss wieder addiert werden. An dem
;Wert im Register COUNTER kann man nun ablesen welche dezimale Wertigkeit die Stelle hat.
;*******************************************************************************************
B2D
	_BANK_0
	movf Ux100_H, W		;Spannungswert fr Subtraktion kopieren 
	movwf CALC_H_1
	movf Ux100_L, W
	movwf CALC_L_1

	clrf COUNTER			;Zhler, der die Anzahl der Subtraktionen zhlt
subtrahieren_H
	movlw d'100'			;Es sollen jeweils 100 von Wert abgezogen werden
	movwf CALC_L_2
	clrf CALC_H_2			;CALC_H_2 wird fr die Berechnung nicht bentigt -> Wert = 0
	call Sub16				;subtrahiere 100 von dem Spannungswert in den Registern Ux100_H/L
	btfsc STATUS, C		;prfen, ob das Ergebnis positiv oder negativ ist
	goto ergebnis_Pos_H	;Ergebnis positiv -> es mssen nochmal 100 abgezogen werden
	goto ergebnis_Neg_H	;Ergebnis negativ -> die Hunderterstelle ist nun bekannt
ergebnis_Pos_H
	incf COUNTER, F		;Zhler erhhen und nochmal 100 abziehen
	goto subtrahieren_H
ergebnis_Neg_H
	movlw d'100'			;es wurden 100 zuviel abgezogen und mssen wieder addiert werden
	movwf CALC_L_2
	clrf CALC_H_2
	call Add16				;es wird 100 addiert
	movf COUNTER, W		;in Register COUNTER steht die Wertigkeit der Hunderterstelle
	movwf DISP_HUNDERTER	;Register CONTER nach DISP_HUNDERTER kopieren
	
	clrf COUNTER			;Wert in COUNTER fr die nchste Berechnung auf 0 zurcksetzen
subtrahieren_Z	
	movlw d'10'				;von dem Spannungswert werden jeweils 10 abgezogen
	movwf CALC_L_2
	clrf CALC_H_2			;CALC_H_2 wird fr die Berechnung nicht bentigt -> Wert = 0
	call Sub16				;subtrahiere 10 von dem verbleibenden Spannungswert
	btfsc STATUS, C		;prfen, ob das Ergebnis positiv oder negativ ist
	goto ergebnis_Pos_Z	;Ergebnis positiv -> es mssen nochmal 10 abgezogen werden
	goto ergebnis_Neg_Z	;Ergebnis negativ -> die Zehnerstelle ist nun bekannt
ergebnis_Pos_Z
	incf COUNTER, F		;Zhler erhhen und nochmal 10 abziehen
	goto subtrahieren_Z
ergebnis_Neg_Z
	movlw d'10'				;es wurden 10 zuviel abgezogen und mssen wieder addiert werden
	movwf CALC_L_2
	clrf CALC_H_2
	call Add16				;es wird 10 addiert
	movf COUNTER, W		;in Register COUNTER steht die Wertigkeit der Zehnerstelle
	movwf DISP_ZEHNER		;die Zehnerstelle von COUNTER nach DISP_ZEHNER kopieren
	
	movf CALC_L_1, W		;um die Einerstelle zu erhalten muss nur noch der Registerinhalt
	movwf DISP_EINER		;von CALC_L_1 nach DISP_EINER kopiert werden

	return

;*******************************************************************************************
;Daten von AD-Wandlung konvertieren
;Die Daten liegen nach der AD-Wandlung als 10 bit Wert in den Registern ADRESH und ADRESL
;Die Werte mssen right justified im AD-Register stehen
;Das Unterprogramm setzt eine Referenzspannung von 5V voraus und berechnet den
;100fachen Spannungswert des AD Ergebnisses. Dies ist ntig, da man nur mit ganzzahligen
;Werten rechnen kann. Das Ergebnis hat dann zwei Nachkommastellen.
;Da der PIC keinen Multiplikationsbefehl zur verfgung stellt, muss dieser durch Additions-
;und Schiebebefehle realisiert werden. Die Formel fr die Berechnung der Spannung lautet:
; U*100 = AD-Wert * 5/1024 * 100 = AD-Wert * (25/64 + 25/256)
;Das Ergebnis von AD-Wert*25 steht in den Registern ADC_Lx25 und ADC_Hx64
;Das Ergebnis von AD-Wert*25/64 steht in den Registern ADC_Lx25_64 und ADC_Hx25_64
;Das Ergebnis von AD-Wert*25/256 steht in den Registern ADC_Lx25_256 und ADC_Hx25_256
;Am Ende des Unterprogramms steht der Spannungswert*100 in den Registern
;ADC_Lx25_256 und ADC_Hx25_256
;*******************************************************************************************
AD_konvertieren
	_BANK_0
	movf ADRESH, W			;AD-Wert in das W-Register kopieren
	;movlw 0x01				;Hiermit kann man die AD-Werte durch Testwerte berschreiben
	movwf ADC_H				;Highbyte des AD-Wertes in das Register ADC_H kopieren
	movwf ADC_Hx25			;Nochmals in das Register ADC_Hx25 kopieren
	_BANK_1
	movf ADRESL, W			;AD-Wert in das W-Register kopieren
	_BANK_0
	;movlw 0xE0				;Hiermit kann man die AD-Werte durch Testwerte berschreiben
	movwf ADC_L				;Lowbyte des AD-Wertes in das Register ADC_L kopieren
	movwf ADC_Lx25			;Nochmals in das Register ADC_Lx25 kopieren
	
	;Multiplikation mit 25
	bcf STATUS, C			;Carryflag vor dem Verschieben lschen
	rlf ADC_Lx25, F 		;Multiplikation mit 2 durch Verschiebung nach links
	rlf ADC_Hx25, F
	
	movf ADC_L, W			;Den Wert nochmals addieren (Multiplikation mit 3)
	addwf ADC_Lx25, F
	btfsc STATUS, C
	incf ADC_Hx25, F
	movf ADC_H, W
	addwf ADC_Hx25, F
	
	bcf STATUS, C			;Carryflag vor dem Verschieben lschen
	rlf ADC_Lx25, F		;nach links schieben = Multiplikation mit 6
	rlf ADC_Hx25, F

	bcf STATUS, C			;Carryflag vor dem Verschieben lschen
	rlf ADC_Lx25, F		;nach links schieben = Multiplikation mit 12
	rlf ADC_Hx25, F

	bcf STATUS, C			;Carryflag vor dem Verschieben lschen	
	rlf ADC_Lx25, F		;nach links schieben = Multiplikation mit 24
	rlf ADC_Hx25, F
	
	movf ADC_L, W			;Den Wert nochmals addieren (Multiplikation mit 25)
	addwf ADC_Lx25, F		;Lowbyte der Multiplikation mit 25 steht jetzt in ADC_Lx25
	btfsc STATUS, C
	incf ADC_Hx25, F
	movf ADC_H, W
	addwf ADC_Hx25, F		;Highbyte der Multiplikation mit 25 steht jetzt in ADC_Hx25
	
	movf ADC_Hx25, W		;Register ADC_Hx25 nach ADC_Hx25_64 kopieren
	movwf ADC_Hx25_64
	movf ADC_Lx25, W		;Register ADC_Lx25 nach ADC_Lx25_64 kopieren
	movwf ADC_Lx25_64

	;AD-Wert mal 25 durch 64 dividieren (6 mal nach rechts schieben)
	bcf STATUS, C			;Carryflag vor dem Verschieben lschen
	rrf ADC_Hx25_64, F	;Division durch 2 durch Verschiebung nach rechts
	rrf ADC_Lx25_64, F

	bcf STATUS, C			;Carryflag vor dem Verschieben lschen
	rrf ADC_Hx25_64, F	;nach rechts schieben = Division durch 4
	rrf ADC_Lx25_64, F

	bcf STATUS, C			;Carryflag vor dem Verschieben lschen
	rrf ADC_Hx25_64, F	;nach rechts schieben = Division durch 8
	rrf ADC_Lx25_64, F

	bcf STATUS, C			;Carryflag vor dem Verschieben lschen
	rrf ADC_Hx25_64, F	;nach rechts schieben = Division durch 16
	rrf ADC_Lx25_64, F

	bcf STATUS, C			;Carryflag vor dem Verschieben lschen
	rrf ADC_Hx25_64, F	;nach rechts schieben = Division durch 32
	rrf ADC_Lx25_64, F

	bcf STATUS, C			;Carryflag vor dem Verschieben lschen
	rrf ADC_Hx25_64, F	;nach rechts schieben = Division durch 64
	rrf ADC_Lx25_64, F
	
	movf ADC_Hx25_64, W	;Register ADC_Hx25_64 nach ADC_Hx25_256 kopieren
	movwf ADC_Hx25_256
	movf ADC_Lx25_64, W	;Register ADC_Lx25_64 nach ADC_Hx25_256 kopieren
	movwf ADC_Lx25_256
	
	;AD-Wert mal 25 durch 256 dividieren
	;Ergebnis der Division durch 64 noch 2 mal nach rechts schieben
	bcf STATUS, C			;Carryflag vor dem Verschieben lschen
	rrf ADC_Hx25_256, F	;nach rechts schieben = Division durch 128
	rrf ADC_Lx25_256, F

	bcf STATUS, C			;Carryflag vor dem Verschieben lschen
	rrf ADC_Hx25_256, F	;nach rechts schieben = Division durch 256
	rrf ADC_Lx25_256, F
	
	;die Werte ADCx25/64 und ADCx25/256 addieren
	movf ADC_Lx25_64, W
	addwf ADC_Lx25_256, W
	movwf Ux100_L				;Lowbyte des Spannungswertes mal 100
	btfsc STATUS, C
	incf ADC_Hx25_256, F
	movf ADC_Hx25_64, W
	addwf ADC_Hx25_256, W
	movwf Ux100_H				;Highbyte des Spannungswertes mal 100

	return

;*******************************************************************************************
; Es wird ein Kommando/Befehl an das Display gesendet.
; Das zu sendende Kommando muss im W-Register stehen, bevor das Unterprogramm
; aufgerufen wird.
; Es werden zwei Register bentigt:
; DISP_SHIFT_REG:	Befehl, der ausgegeben wird
; DISP_SHIFT_CNT: Anzahl der auszugebenden bits
;*******************************************************************************************
SchreibeDispKommando
	movwf DISP_SHIFT_REG		;Register mit den Daten, die ausgegeben werden sollen
	bcf DISP_CSB				;CSB auf Low setzen, damit das Display Daten empfangen kann 
	bcf DISP_RS					;RS auf Low setzen, da es sich um ein Kommando handelt
	bcf DISP_CLK				;Takt auf Low setzen, damit die Daten bei der steigenden
									;Flanke bernommen werden knnen
	movlw d'8'					;Zhler fr die Anzahl der zu bertragenden bits (=8)
	movwf DISP_SHIFT_CNT
shiftKommando
	rlf DISP_SHIFT_REG, F	;Bitverschiebung nach links. Das Carrybit wird am Pin SI
									;ausgegeben
	btfsc STATUS, C			;Wenn das Carryflag 0 ist, wird in der Pin SI mit dem Befehl
									;bcf DISP_SI auf Lowpegel gesetzt
	bsf DISP_SI					;Das Carryflag ist 1 -> setze Pin SI auf Highpegel
	btfss STATUS, C
	bcf DISP_SI
	nop							;eine kleine Wartezeit, um die bertragungsgeschwindigkeit
									;an das Display anzupassen
	bsf DISP_CLK				;mit der positiven Flanke bernimmt das Display die Daten
	nop
	bcf DISP_CLK				;negative Flanke des Taktes
	decfsz DISP_SHIFT_CNT, F	;Bitzhler um 1 verringern
	goto shiftKommando		;Wenn noch nicht 8 bits ausgegeben wurden, wird zum Label
									;shiftKommando gesprungen.
	bcf DISP_SI					;alle Daten bertragen -> Pin SI auf Low setzen
	bcf DISP_RS					;Pin RS auf Low setzen
	_DELAY_TMR1_US	d'30'		;geforderte Wartezeit des Displays

	return						;Rcksprung aus dem Unterprogramm

;*******************************************************************************************
; Es wird ein Datum an das Display gesendet.
; Das zu sendende Zeichen muss im W-Register stehen, bevor das Unterprogramm
; aufgerufen wird.
; Es werden zwei Register bentigt:
; DISP_SHIFT_REG:	Befehl, der ausgegeben wird
; DISP_SHIFT_CNT: Anzahl der auszugebenden bits
;*******************************************************************************************
SchreibeDispDaten
	movwf DISP_SHIFT_REG		;Das W-Register wird in das Reg. DISP_SHIFT_REG kopiert
	bcf DISP_CSB				;CSB auf Low setzen, damit das Display Daten empfangen kann 
	bsf DISP_RS					;RS auf High setzen, da es sich um ein Zeichen handelt
	bcf DISP_CLK				;Takt auf Low setzen, damit die Daten bei der steigenden
									;Flanke bernommen werden knnen
	movlw d'8'					;Es sollen 8 bits bertragen werden
	movwf DISP_SHIFT_CNT		;Zhler fr die bertragenen bits
shiftDaten
	rlf DISP_SHIFT_REG, F	;Das Zeichen wird nach links verschoben. Das Carryflag wird
									;an den Ausgang weitergegeben
	btfsc STATUS, C			;Prfen, welchen Wert das Carryflag hat
	bsf DISP_SI					;Carryflag = 1 -> Pin SI auf Highpegel setzen
	btfss STATUS, C			;Nochmal prfen, da immer nur ein Befehl bersprungen wird
	bcf DISP_SI					;Carryflag = 0 -> Pin SI auf Lowpegel setzen
	nop							;kurze Wartezeit, um die bertragungsgeschwindigkeit zu reduzieren
	bsf DISP_CLK				;mit dieser steigenden Flanke werden die Daten bernommen
	nop							;kurze Wartezeit
	bcf DISP_CLK				;negative Flanke des Taktes
	decfsz DISP_SHIFT_CNT, F	;ein bit wurde bertragen -> Zhler um 1 verringern
	goto shiftDaten			;nchstes bit bertragen
	bcf DISP_RS					;bertragung beendet Pin RS auf Low zurcksetzen
	bcf DISP_SI					;Pin SI auf Low zurcksetzen
	_DELAY_TMR1_US	d'30'		;geforderte Wartezeit (min. 26,3 us)

	return						;Rcksprung aus dem Unterprogramm

;*******************************************************************************************
;	Sendet ein Zeichen ber die serielle Schnittstelle
;	Das Zeichen muss im W-Register stehen
;*******************************************************************************************
SendeZeichen
	_BANK_1
	bsf TXSTA, TXEN			;freischalten fr die bertragung
	_BANK_0
	movwf TXREG
	_BANK_1						; warten, bis die Daten gesendet wurden
	btfss TXSTA, TRMT
	goto $-1
	_BANK_0
	return

	END                		;Ende des Programms







